@@ -135,9 +135,49 @@ def wavesol(xpix,wave,order=3,xr=None):
     return coef,ww
         
 
-def trace(im,yestimate=None,yorder=2,sigorder=4,step=10):
-    """ Trace the spectrum.  Spectral dimension is assumed to be on the horizontal axis."""
-    ny,nx = im.shape
+def trace(im,yestimate=None,yorder=2,sigorder=4,step=10,spectral_axis=1):
+    """
+    Trace the spectrum.  Spectral dimension is assumed to be on the horizontal axis.
+
+    Parameters
+    ----------
+    im : numpy array
+       The input 2D image.
+    yestimate : float, optional
+       The initial estimate of the central Y (spatial dimension) position of the trace.  Default
+          is to calculate using a median cut.
+    yorder : int, optional
+       Polynomial order to use for fitting the trace position as a function of column.  Default is 2.
+    sigorder : int, optional
+       Polynomial order to use for fitting the Gaussian sigma as a function of column.  Default is 4.
+    step : int, optional
+       Stepsize to take in the spectral dimension when tracing the spectrum.  Default is 10 pixels.
+    spectral_axis : int, optional
+       The spectral axis.  Default is 1.
+
+    Returns
+    -------
+    tcat : table
+       Table of Gaussian fits when stepping along in columns and tracing the spectrum.
+    ypars : numpy array
+       Polynomial coefficients of the trace.
+    sigpars : numpy array
+       Polynomial coefficients of the Gaussian sigma.
+    mcat : table
+       Table of model x, y and sigma values along the spectrum.
+
+    Example
+    -------
+
+    tcat,ypars,sigpars,mcat = trace(im)    
+
+    """
+
+    spec = np.copy(im)  # internal copy
+    if spectral_axis==0:  # transpose
+        spec = spec.T
+    ny,nx = spec.shape
+    y = np.arange(ny)
     if yestimate is None:
         ytot = np.sum(im,axis=1)
         yestimate = np.argmax(ytot)
@@ -146,11 +186,17 @@ def trace(im,yestimate=None,yorder=2,sigorder=4,step=10):
     smim = ndimage.uniform_filter1d(im, 50, 1)
     nstep = nx//step
     # Loop over the columns in steps and fit Gaussians
-    tcat = np.zeros(nstep,dtype=np.dtype([('x',float),('pars',float,4)]))
+    tcat = np.zeros(nstep,dtype=np.dtype([('x',float),('amp',float),('y',float),('sigma',float),
+                                          ('pars',float,4),('perror',float,4)]))
     for i in range(nstep):
         pars,cov = dln.gaussfit(y[yestimate-10:yestimate+10],im[yestimate-10:yestimate+10,step*i+step//2])
+        perror = np.sqrt(np.diag(cov))
         tcat['x'][i] = step*i+step//2
+        tcat['amp'][i] = pars[0]
+        tcat['y'][i] = pars[1]
+        tcat['sigma'][i] = pars[2]
         tcat['pars'][i] = pars
+        tcat['perror'][i] = perror        
     # Fit polynomail to y vs. x and gaussian sigma vs. x
     ypars = np.polyfit(tcat['x'],tcat['pars'][:,1],yorder)
     sigpars = np.polyfit(tcat['x'],tcat['pars'][:,2],sigorder)
@@ -162,29 +208,106 @@ def trace(im,yestimate=None,yorder=2,sigorder=4,step=10):
     mcat['sigma'] = np.poly1d(sigpars)(xx)
     return tcat, ypars, sigpars,mcat
 
-def boxcar(im):
-    """ Boxcar extract the spectrum"""
-    ny,nx = im.shape
-    ytot = np.sum(im,axis=1)
-    yest = np.argmax(ytot)
-    # Background subtract
-    yblo = int(np.maximum(yest-50,0))
-    ybhi = int(np.minimum(yest+50,ny))
-    med = np.median(im[yblo:ybhi,:],axis=0)
-    medim = np.repeat(med,ny).reshape(ny,nx)
-    subim = im.copy()-medim
+def boxsum(im,ylo,yhi):
+    """ Helper function for boxcar extraction."""
+    y0 = np.min(ylo).astype(int)
+    y1 = np.max(yhi).astype(int)
     # Sum up the flux
-    ylo = int(np.maximum(yest-20,0))
-    yhi = int(np.minimum(yest+20,ny))
-    flux = np.sum(subim[ylo:yhi,:],axis=0)
+    subim = im[y0:y1,:]
+    ny,nx = subim.shape
+    xx,yy = np.meshgrid(np.arange(nx),np.arange(ny))
+    mask = (yy>=(ylo-y0)) & (yy<=(yhi-y0))
+    flux = np.sum(mask*subim,axis=0)
     return flux
+    
+    
+def boxcar(im,ytrace=None,width=20,backlo=None,backhi=None):
+    """
+    Boxcar extract the spectrum
+
+    Parameters
+    ----------
+    im : numpy array
+      Image from which to extract the spectrum.
+    ytrace : numpy array, optional
+      The y (spatial) position of the trace as a function of column.
+    width : int, optional
+      The half-width of the spectrum in the spatial dimension to extract.
+    backlo : tuple, optional
+       The lower and upper offsets (relative to the trace) for the lower
+         background region (e.g., (-50,-40)).  Default is None.
+    backhi : tuple, optional
+       The lower and upper offsets (relative to the trace) for the upper.
+         background region (e.g., (40,50)).  Default is None.
+
+    Returns
+    -------
+    out : table
+       Output table with flux and background values.
+
+    Example
+    -------
+
+    out = boxcar(im)
+
+    """
+    
+    ny,nx = im.shape
+    # Get median trace position
+    if ytrace is None:
+        ytot = np.sum(im,axis=1)
+        yest = np.argmax(ytot)
+        ytrace = np.zeros(nx,float)+yest
+        
+    # Start output
+    dt = np.dtype([('x',float),('ytrace',float),('sumflux',float),('background',float),
+                 ('backlo',float),('backhi',float),('flux',float)])
+    out = np.zeros(nx,dtype=dt)
+    out['x'] = np.arange(nx)
+    out['ytrace'] = ytrace
+    
+    # Sum up the flux
+    ylo = np.maximum(ytrace-width,0).astype(int)
+    yhi = np.minimum(ytrace+width,ny).astype(int)
+    flux = boxsum(im,ylo,yhi)
+
+    out['sumflux'] = flux
+    # Background
+    bflux = None
+    if backlo is not None:
+        bloylo = np.maximum(ytrace+backlo[0],0).astype(int)
+        bloyhi = np.maximum(ytrace+backlo[1],0).astype(int)
+        bloflux = boxsum(im,bloylo,bloyhi)
+        bflux = bloflux
+        out['backlo'] = bloflux
+    if backhi is not None:
+        bhiylo = np.minimum(ytrace+backhi[0],ny).astype(int)
+        bhiyhi = np.minimum(ytrace+backhi[1],ny).astype(int)
+        bhiflux = boxsum(im,bhiylo,bhiyhi)
+        bflux = bhiflux
+        out['backhi'] = bhiflux        
+    # Average backgrounds
+    if backlo is not None and backhi is not None:
+       bflux = 0.5*(bloflux+bhiflux) 
+
+    # Final flux
+    if bflux is not None:
+        out['background'] = bflux
+        out['flux'] = flux-bflux
+    else:
+        out['flux'] = flux        
+        
+    return out
+
 
 def gaussline(x, amp, cen, sigma, const=0):
     """1-D gaussian: gaussian(x, amp, cen, sig)"""
     return amp * np.exp(-(x-cen)**2 / (2*sigma**2)) + const
 
 def fitlines(x,y,err=None,nsig=5):
-    """ Automatically detect emission lines and fit Gaussians to them."""
+    """
+    Automatically detect emission lines and fit Gaussians to them.
+    """
 
     n = len(x)
     sm = 101
@@ -260,17 +383,49 @@ def linefit(x,y,initpar,bounds,err=None):
     return curve_fit(gline, x, y, p0=line_initpar, bounds=line_bounds, sigma=err)
 
 
-def extract(im,imerr=None,mcat=None,nobackground=False):
-    """ Extract a spectrum"""
+def extract(im,imerr=None,mcat=None,nobackground=False,verbose=False):
+    """
+    Extract a spectrum
+
+    Parameters
+    ----------
+    im : numpy array
+       The 2D image array from which to extract the spectra
+    imerr : numpy array, optional
+       Uncertainty array for im.
+    mcat : table
+       Table containing the trace information returned from trace().
+         This must (at last) contain the "y" and "sigma" columns.
+         If this is not input, then trace() will be run on the image.
+    nobackground : boolean, optional
+       Do not subtract the background.  This is useful when extracting
+         arc lamp data from longslit spectra.  Default is False.
+    verbose : boolean, optional
+       Verbose output to the screen.  Default is False.
+
+    Returns
+    -------
+    cat : table
+       Table containing the extracted spectrum and uncertainties.
+
+    Example
+    -------
+
+    cat = extract(im,imerr,mcat)
+
+    """
+    
     ny,nx = im.shape
     x = np.arange(nx)
     y = np.arange(ny)
     # No trace information input, get it
     if mcat is None:
-        tcat,ypars,sigpars,mcat=trace(im)
-    # Loop over the columns and get the flux using the trace information
+        tcat,ypars,sigpars,mcat = trace(im)
+        
+    # Initialize the output
     cat = np.zeros(nx,dtype=np.dtype([('x',int),('pars',float,2),('perr',float,2),
                                       ('flux',float),('fluxerr',float)]))
+    # Loop over the columns and get the flux using the trace information
     for i in range(nx):
         line = im[:,i].flatten()
         if imerr is not None:
@@ -288,9 +443,11 @@ def extract(im,imerr=None,mcat=None,nobackground=False):
         # Only fit the region right around the peak
         y0 = int(np.maximum(ycen-50,0))
         y1 = int(np.minimum(ycen+50,ny))
-        bnds = ([0,ycen-1e-4,ysigma-1e-4,0],[1.5*ht0,ycen,ysigma,1.5*initpar[3]])
+        bnds = ([0,ycen-1e-7,ysigma-1e-7,np.min(line)],[1.5*ht0,ycen+1e-7,ysigma+1e-7,initpar[3]+3*dln.mad(line)])
         if nobackground is True:
-            bnds = ([0,ycen-1e-4,ysigma-1e-4,0],[1.5*ht0,ycen,ysigma,0.1])
+            initpar[3] = 0.0
+            bnds = ([0,ycen-1e-4,ysigma-1e-4,-1e-7],[1.5*ht0,ycen,ysigma,1e-7])
+        # We could probably use linear algebra to figure this out more quickly
         pars,cov = linefit(y[y0:y1],line[y0:y1],initpar=initpar,bounds=bnds,err=lineerr[y0:y1])
         perr = np.sqrt(np.diag(cov))
         # Gaussian area = ht*wid*sqrt(2*pi)
@@ -301,37 +458,74 @@ def extract(im,imerr=None,mcat=None,nobackground=False):
         cat['perr'][i] = perr
         cat['flux'][i] = flux
         cat['fluxerr'][i] = fluxerr
+        if verbose:
+            print(i,pars)
     return cat
 
 def emissionlines(spec,thresh=None):
-    """Measure the emission lines in an arc lamp spectrum. """
+    """
+    Measure the emission lines in an arc lamp spectrum.
+
+    Parameters
+    ----------
+    spec : numpy array
+       The input spectrum in which to detect emission peaks.
+    thresh : float, optional
+       The threshold to use for detecting peaks.  The default
+          is 5 sigma above the background.
+
+    Returns
+    -------
+    lines : table
+        Output table with measurements of peaks.
+    model : numpy array
+        Model spectrum of the peaks.
+
+    Example
+    -------
+
+    lines, model = spec.emissionlines(flux)
+
+    """
     nx = len(spec)
     x = np.arange(nx)
+
+    # Remove background
+    med = medfilt(spec,101)
+    subspec = spec-med
     
     # Threshold
     if thresh is None:
-        thresh = np.min(spec) + (np.max(spec)-np.min(spec))*0.05
-    
+        #thresh = (np.max(subspec)-np.min(subspec))*0.05
+        sig = dln.mad(subspec)
+        thresh = 5*sig
+        
     # Detect the peaks
-    sleft = np.hstack((0,spec[0:-1]))
-    sright = np.hstack((spec[1:],0))
-    peaks, = np.where((spec>sleft) & (spec>sright) & (spec>thresh))
+    sleft = np.hstack((0,subspec[0:-1]))
+    sright = np.hstack((subspec[1:],0))
+    peaks, = np.where((subspec>sleft) & (subspec>sright) & (subspec>thresh))
     npeaks = len(peaks)
     print(str(npeaks)+' peaks found')
     
     # Loop over the peaks and fit them with Gaussians
     gcat = np.zeros(npeaks,dtype=np.dtype([('x0',int),('x',float),('xerr',float),('pars',float,4),('perr',float,4),
-                                           ('flux',float),('fluxerr',float)]))
-    resid = spec.copy()
+                                           ('flux',float),('fluxerr',float),('amp',float),('sigma',float),('success',bool)]))
+    resid = subspec.copy()
     gmodel = np.zeros(nx)
     for i in range(npeaks):
         x0 = peaks[i]
         xlo = np.maximum(x0-6,0)
         xhi = np.minimum(x0+6,nx)
-        initpar = [spec[x0],x0,1,0]
+        initpar = [subspec[x0],x0,1,0]
         bnds = ([0,x0-3,0.1,0],[1.5*initpar[0],x0+3,10,1e4])
-        pars,cov = dln.gaussfit(x[xlo:xhi],spec[xlo:xhi],initpar,bounds=bnds,binned=True)
-        perr = np.sqrt(np.diag(cov))
+        try:
+            pars,cov = dln.gaussfit(x[xlo:xhi],subspec[xlo:xhi],initpar,bounds=bnds,binned=True)
+            perr = np.sqrt(np.diag(cov))
+            gcat['success'][i] = True
+        except:
+            pars = initpar
+            perr = np.copy(initpar)*0.0
+            gcat['success'][i] = False            
         gmodel1 = dln.gaussian(x[xlo:xhi],*pars)
         gmodel[xlo:xhi] += (gmodel1-pars[3])
         resid[xlo:xhi] -= (gmodel1-pars[3])
@@ -345,9 +539,58 @@ def emissionlines(spec,thresh=None):
         gcat['perr'][i] = perr
         gcat['flux'][i] = flux
         gcat['fluxerr'][i] = fluxerr
+        gcat['amp'][i] = pars[0]
+        gcat['sigma'][i] = pars[2]
         
     return gcat, gmodel
 
+def matchlines(w1,w2,dcr=0.5):
+    """
+    Match up two lists of lines within a 0.5 toleration.
+    
+    Parameters
+    ----------
+    w1 : numpy array
+       First array of wavelengths.
+    w2 : numpy array
+       Second array of wavelengths.
+    dcr : float
+       The matching radius.
+
+    Returns
+    -------
+    ind1 : numpy array
+       Array of indices into w1 for matches.  None if no matches.
+    ind2 : numpy array
+       Array of indices into w2 for matches.  None if no matches.
+
+    Example
+    -------
+    
+    ind1,ind2 = matchlines(w1,w2,2.0)
+
+    """
+    match = np.zeros(len(w1),bool)
+    wdiff = np.zeros(len(w1),float)
+    wmatch = np.zeros(len(w1),np.float64)
+    xmatch = np.zeros(len(w1),float)
+    indmatch = np.zeros(len(w1),int)-1
+    # Loop over w2
+    for i in range(len(w1)):
+        dist = np.abs(w1[i]-w2)
+        mindist = np.min(dist)
+        ind = np.argmin(dist)
+        if mindist<dcr:
+            match[i] = True
+            indmatch[i] = ind
+    ind1, = np.where(match==True)
+    print(len(ind1),' matches')
+    if len(ind1)>0:
+        ind2 = indmatch[ind1]
+    else:
+        ind1,ind2 = None,None
+        
+    return ind1,ind2
 
 def continuum(spec,bin=50,perc=60,norder=4):
     """ Derive the continuum of a spectrum."""
